@@ -1,4 +1,4 @@
-# Copyright (c) 2020, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,10 +26,8 @@ from __future__ import unicode_literals
 
 import datetime, re
 
-from django.utils.translation import ugettext_lazy as _
-
 from . import settings
-
+from .compat import gettext_lazy as _
 
 # prevents an import loop with models.py
 HOURLY = 1
@@ -104,6 +102,9 @@ DESCRIBE_SUFFIX_DISCOUNT_PERIOD = \
 DESCRIBE_SUFFIX_DISCOUNT_CURRENCY = \
     "a %(amount)s off"
 
+DESCRIBE_SUFFIX_TARGET_SUBSCRIBER = \
+    "for %(subscriber_full_name)s (%(sync_on)s)"
+
 DESCRIBE_SUFFIX_GROUP_BUY = \
     ", complimentary of %(payer)s"
 
@@ -128,7 +129,9 @@ REGEXES = {
     'plan': r'(?P<plan>\S+)',
     'quantity': r'(?P<quantity>\d+)',
     'refund_type': r'(?P<refund_type>\S+)',
+    'subscriber_full_name': r'(?P<subscriber_full_name>(\S| )+)',
     'subscription': r'(?P<subscriber>\S+):(?P<plan>\S+)',
+    'sync_on': r'(?P<sync_on>\S+)',
     'use_charge': r'(?P<use_charge>\S+)',
     'unlock_event': r'(?P<unlock_event>\S+)',
 }
@@ -271,6 +274,7 @@ def translate_descr_suffix(descr):
         r"(%(nb_periods)s %(period_name)s free)?( and )?"\
         r"(a %(amount)s off)?"\
         r"(, complimentary of %(payer)s)?"\
+        r"(\s*for %(subscriber_full_name)s \(%(sync_on)s\))?"\
         r"( \(code: %(code)s\))?" % REGEXES
     look = re.match(pat, descr_suffix)
     if look:
@@ -281,28 +285,35 @@ def translate_descr_suffix(descr):
         sep = ""
         percent = look.group('percent')
         if percent:
-            descr_suffix += sep + _("a %(percent)s discount") % {
+            descr_suffix += sep + _(DESCRIBE_SUFFIX_DISCOUNT_PERCENTAGE) % {
                 'percent': percent}
             sep = _(" and ")
         nb_periods = look.group('nb_periods')
         period_name = look.group('period_name')
         if nb_periods and period_name:
             nb_periods = int(nb_periods)
-            descr_suffix += sep + _("%(nb_periods)d %(period_name)s free") % {
+            descr_suffix += sep + _(DESCRIBE_SUFFIX_DISCOUNT_PERIOD) % {
                 'nb_periods': nb_periods,
                 'period_name': translate_period_name(period_name, nb_periods)}
             sep = _(" and ")
         amount = look.group('amount')
         if amount:
-            descr_suffix += sep + _("a %(amount)s off") % {
+            descr_suffix += sep + _(DESCRIBE_SUFFIX_DISCOUNT_CURRENCY) % {
                 'amount': amount}
             sep = _(" and ")
         payer = look.group('payer')
         if payer:
             descr_suffix += _(", complimentary of %(payer)s") % {'payer': payer}
+        subscriber_full_name = look.group('subscriber_full_name')
+        sync_on = look.group('sync_on')
+        if subscriber_full_name and sync_on:
+            descr_suffix += " " + _(DESCRIBE_SUFFIX_TARGET_SUBSCRIBER) % {
+                'subscriber_full_name': subscriber_full_name,
+                'sync_on': sync_on
+            }
         code = look.group('code')
         if code:
-            descr_suffix += _(" (code: %(code)s)") % {'code': code}
+            descr_suffix += _(DESCRIBE_SUFFIX_COUPON_APPLIED) % {'code': code}
 
     if descr_suffix:
         descr += " - %s" % descr_suffix
@@ -310,7 +321,7 @@ def translate_descr_suffix(descr):
 
 
 def describe_buy_periods(plan, ends_at, nb_periods, discount_by_types=None,
-                         coupon=None, full_name=None):
+                         coupon=None, cart_item=None, full_name=None):
     #pylint:disable=too-many-arguments
     descr = DESCRIBE_BUY_PERIODS % {
         'plan': plan,
@@ -320,9 +331,17 @@ def describe_buy_periods(plan, ends_at, nb_periods, discount_by_types=None,
     sep = ""
     descr_suffix = ""
 
-    if not coupon and full_name:
+    # triggered by `new_subscription_order` through the renewals
+    # process (amount is None).
+    # XXX It seems to be able to figure out which account was billed
+    # when receiving multiple receipts.
+    if full_name:
         descr_suffix += "%s" % full_name
 
+    if coupon and not discount_by_types:
+        discount_by_types = {}
+        discount_by_types[coupon.discount_type] = coupon.discount_value
+
     if discount_by_types:
         discount_amount = discount_by_types.get(DISCOUNT_PERCENTAGE)
         if discount_amount:
@@ -344,13 +363,25 @@ def describe_buy_periods(plan, ends_at, nb_periods, discount_by_types=None,
 
     if coupon:
         if coupon.code.startswith('cpn_'):
-            if full_name:
+            # We have switched the full_name from the target subscriber
+            # to the group buyer in `execute_order`.
+            if cart_item and cart_item.full_name:
                 descr_suffix += DESCRIBE_SUFFIX_GROUP_BUY % {
                     'payer': full_name}
         else:
             descr_suffix += DESCRIBE_SUFFIX_COUPON_APPLIED % {
                 'code': coupon.code}
 
+    if cart_item and cart_item.sync_on:
+        # We also set sync_on to None in `execute_order`, so both
+        # above DESCRIBE_SUFFIX_GROUP_BUY and here cannot happen
+        # at the same time.
+        if descr_suffix:
+            descr_suffix += " "
+        descr_suffix += DESCRIBE_SUFFIX_TARGET_SUBSCRIBER % {
+            'subscriber_full_name': cart_item.full_name,
+            'sync_on': cart_item.sync_on}
+
     if descr_suffix:
         descr += " - %s" % descr_suffix
     return descr
